#!/usr/local/bin/perl
#
# Setup console moused

#
# Copyright (c) 1997 
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 


use lib '/usr/home/berend/FreEasy/lib';
use CGI;
use access_rcconf;

$form = new CGI;

#
# Check if FreeBSD version 2.2.1 is installed, because from this version
# is moused supported.
#

$Release = `uname -r`;

if ($Release !~ /2\.1\./) {

    # get settings from /etc/rc.conf
    %vars = ();
    (%vars) = rcconf_get_some_vars('moused_type', 'moused_port', 'moused_flags');

    # parse moused flags
    $baudrate = 1200;
    $LowerDTR = 0;
    $LowerRTS = 0;
    $ChordMiddle = 0;
    @flags = split / /, $vars{'moused_flags'};
    while ($_ = $flags[0], /^-/) {
	shift @flags;
	last if /^--$/;
	if (/^-s/) { $baudrate = $flags[0]; shift @flags; }
	elsif (/^-S/) { $baudrate = 9600; }
	elsif (/^-D/) { $LowerDTR = 1; }
	elsif (/^-R/) { $LowerRTS = 1; }
	elsif (/^-c/) { $ChordMiddle = 1; }
	else { 
	    # forward to next dash
	    while ($_ = $flags[0], not (/^-/)) { 
		shift @flags; last if ($_ eq ''); 
	    } 
	}
    }

    print $form->header;
    print $form->start_html(
      -title=>'Setup moused',
      -BGCOLOR=>'white');

    print "<FORM METHOD=\"GET\" ACTION=\"http:changemouse.pl\">\n";
    print "<P>This is the type of mouse you would like to use for doing\n";
    print "cut/paste selection on the console.\n";
    print "<P>It should also be noted that while this is enabled, any\n";
    print "other client of the mouse (such as an X server) should access\n";
    print "the mouse through the virtual mouse device\n";
    print "<tt>/dev/sysmouse</tt> and configure it as a mousesystems type\n";
    print "mouse, since all mouse data is converted to this single\n";
    print "canonical format when using ";
    print "<a href=http://www.freebsd.org/cgi/man.cgi?query=moused>moused</a>.\n";

    print "<CENTER>\n";

    print "<H1>moused options:</H1>\n";

# mouse type
    print "<H3>Mouse type</H3>\n";

    print "<TABLE>\n";

    print "<TD><SELECT NAME=\"moused_type\">\n";

    print '<OPTION VALUE="NO"', 
      ($vars{'moused_type'} eq "NO") ? 'SELECTED' : '', 
      ">Help, I'm afraid of mice\n";

    print '<OPTION VALUE="microsoft"', 
      ($vars{'moused_type'} eq "microsoft") ? 'SELECTED' : '', 
      ">Microsoft mouse\n";

    print '<OPTION VALUE="microsoft"', 
      ($vars{'moused_type'} eq "mousesystems") ? 'SELECTED' : '', 
      ">Mouse systems Corp mouse\n";

    print '<OPTION VALUE="mmseries"', 
      ($vars{'moused_type'} eq "mmseries") ? 'SELECTED' : '', 
      ">MM Series mouse\n";

    print '<OPTION VALUE="logitech"', 
      ($vars{'moused_type'} eq "logitech") ? 'SELECTED' : '', 
      ">Logitech mouse\n";

    print '<OPTION VALUE="busmouse"', 
      ($vars{'moused_type'} eq "busmouse") ? 'SELECTED' : '', 
      ">A bus mouse\n";

    print '<OPTION VALUE="mouseman"', 
      ($vars{'moused_type'} eq "mouseman") ? 'SELECTED' : '', 
      ">Logitech MouseMan and TrackMan\n";

    print '<OPTION VALUE="ps/2"', 
      ($vars{'moused_type'} eq "ps/2") ? 'SELECTED' : '', 
      ">PS/2 mouse\n";

    print '<OPTION VALUE="mmhittab"', 
      ($vars{'moused_type'} eq "mmhittab") ? 'SELECTED' : '', 
      ">MM HitTablet\n";

    print "</SELECT></TD></TR>\n";
    print "</TABLE>\n";

    print "<P>The Microsoft mouse type is your best choice if you are unsure.</P>\n";

    print "<HR SIZE=\"2\">\n";

# I/O port
    print "<H3>I/O port</H3>\n";

    print "<TABLE>\n";

    if ( ($vars{'moused_port'} eq 'NO') || ($vars{'moused_port'} eq '') ) {
	$vars{'moused_port'} = '/dev/cuaa0';
    }
    print '<TR><TD><input type=radio name="moused_port" VALUE="cuaa0" ',
      ($vars{'moused_port'} =~ /cuaa0/) ? 'CHECKED' : '',
      ">/dev/cuaa0 (COM1)\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD><input type=radio name="moused_port" VALUE="cuaa1" ',
      ($vars{'moused_port'} =~ /cuaa1/) ? 'CHECKED' : '',
      ">/dev/cuaa1 (COM2)\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD><input type=radio name="moused_port" VALUE="cuaa2" ',
      ($vars{'moused_port'} =~ /cuaa2/) ? 'CHECKED' : '',
      ">/dev/cuaa2 (COM3)\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD><input type=radio name="moused_port" VALUE="cuaa3" ',
      ($vars{'moused_port'} =~ /cuaa3/) ? 'CHECKED' : '',
      ">/dev/cuaa3 (COM4)\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD><input type=radio name="moused_port" VALUE="psm0" ',
      ($vars{'moused_port'} =~ /psm0/) ? 'CHECKED' : '',
      ">/dev/psm0 (PS/2 mouse)\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD><input type=radio name="moused_port" VALUE="other" ',
      ($vars{'moused_port'} =~ /(cuaa0|cuaa1|cuaa2|cuaa3|psm0)/) ? '' : 'CHECKED',
      ">other, specify port: \n";
    print "</TD>\n";
    print '</TD><TD><input size=20 name=explicitport  ';
    print "value=\"$vars{'moused_port'}\" maxlength=80></TD></TR>\n";

    print "</TABLE>\n";

    print "<P>Use <tt>/dev/psm0</tt> if you have a mouse which connects ",
      "through a keyboard style connector.</P>\n";

    print "<HR SIZE=\"2\">\n";

# baudrate
    print "<H3>Baudrate</H3>\n";

    print "<TABLE>\n";

    print '<TR><TD></TD><TD><INPUT TYPE="RADIO" NAME="baudrate" VALUE="1200" ',
      ($baudrate == 1200) ? 'CHECKED' : '',
      ">Default baudrate (1200 baud)\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD></TD><TD><INPUT TYPE="RADIO" NAME="baudrate" VALUE="2400" ',
      ($baudrate == 2400) ? 'CHECKED' : '',
      ">2400 baud\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD></TD><TD><INPUT TYPE="RADIO" NAME="baudrate" VALUE="9600" ',
      ($baudrate == 9600) ? 'CHECKED' : '',
      ">9600 baud\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD></TD><TD><INPUT TYPE="RADIO" NAME="baudrate" VALUE="other" ',
      (($baudrate != 1200) && ($baudrate != 2400) && ($baudrate != 9600)) ? 'CHECKED' : '',
      ">other, specify value: \n";
    print '</TD><TD><INPUT SIZE="4" name=explicitbaudrate ';
    print "VALUE=\"$baudrate\" MAXLENGTH=\"4\"></TD></TR>\n";

    print "</TABLE>\n";

    print "<P>The default baudrate is your best choice.</P>\n";

    print "<HR SIZE=\"2\">\n";

# other
    print "<H3>Other options</H3>";
  
    print "<TABLE>\n";

    print '<TR><TD><input type=checkbox ',
      'NAME=chordmiddle VALUE="chordmiddle" ',
      ($ChordMiddle == 1) ? 'CHECKED' : '',
      ">ChordMiddle\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD><input type=checkbox ',
      'NAME=lowerdtr VALUE="lowerdtr" ',
      ($LowerDTR == 1) ? 'CHECKED' : '',
      ">Lower DTR\n";
    print "</TD><TD></TD></TR>\n";

    print '<TR><TD><input type=checkbox ',
      'NAME=lowerrts VALUE="lowerrts" ',
      ($LowerRTS == 1) ? 'CHECKED' : '',
      ">Lower RTS\n";
    print "</TD><TD></TD></TR>\n";
  
    print "</TABLE>\n";

    print "<P>Note: these options are seldom used. ";
    print "ChordMiddle is an option for some 3-button Logitech mouse.</P>\n";

    print "</CENTER>\n";

  print "<P><INPUT TYPE=\"SUBMIT\" VALUE=\"OK\">\n";
  print "<INPUT TYPE=\"RESET\">\n";

  print "</FORM>\n";

}
else {

    #
    # Print form header
    #

    print $form->header;

    print $form->start_html(
       -title=>'No moused support for your system',
       -BGCOLOR=>'white');

    #
    # HTML body
    #

    print "<H3>Sorry, this option is only available if you are running ";
    print "FreeBSD 2.2.1 or later.</H3>\n";

    print "<CENTER><TABLE>\n";

    print "<TR><TD><A HREF=\"http:/FreEasy/cgi-bin/start.pl\">\n";
    print "<IMG SRC=\"/FreEasy/Icons/mainbutton.gif\" ALT=\"BACK TO MAIN";
    print " PROGRAM\" BORDER=0></A></TD>\n";
    print "<TD><A HREF=\"http:/FreEasy/cgi-bin/setup.pl";
    print "?path=Setup&title=Setup+your+System\">\n";
    print "<IMG SRC=\"/FreEasy/Icons/setupbutton.gif\" ALT=\"BACK TO ";
    print "SETUP SCREEN\" BORDER=0></A></TD>\n";

    print "</TABLE></CENTER>\n";

  }

#
# End HTML form
#

print $form->end_html;

# mouse.pl
