#!/usr/local/bin/perl

#
# functions to access the INDEX file
# I didn't hesitate to grab any of jkh's sysinstall code
#
# Created: 1998-02-11

# Copyright (c) 1998 
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 


package access_INDEX;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_INDEX_desc get_INDEX_alias get_INDEX_keyword get_package_ftp_servers);



#
# return known index descriptions
#
sub get_INDEX_desc {

    my %indexdesc = ();

    $indexdesc{"All"} = "All available packages in all categories.";
    $indexdesc{"applications"} = "User application software.";
    $indexdesc{"astro"} = "Applications related to astronomy.";
    $indexdesc{"archivers"} = "Utilities for archiving and unarchiving data.";
    $indexdesc{"audio"} = "Audio utilities - most require a supported sound card.";
    $indexdesc{"benchmarks"} = "Utilities for measuring system performance.";
    $indexdesc{"benchmarking"} = "Utilities for measuring system performance."; 
    $indexdesc{"cad"} = "Computer Aided Design utilities.";
    $indexdesc{"chinese"} = "Ported software for the Chinese market.";
    $indexdesc{"comms"} = "Communications utilities."; 
    $indexdesc{"converters"} = "Format conversion utilities..";
    $indexdesc{"databases"} = "Database software.";
    $indexdesc{"devel"} = "Software development utilities and libraries.";
    $indexdesc{"documentation"} = "Document preparation utilities.";
    $indexdesc{"editors"} = "Common text editors.";
    $indexdesc{"emulat"} = "Utilities for emulating other OS types."; 
    $indexdesc{"games"} = "Various and sundry amusements.";
    $indexdesc{"german"} = "Ported software for the german market.";
    $indexdesc{"graphics"} = "Graphics libraries and utilities.";
    $indexdesc{"japanese"} = "Ported software for the Japanese market.";
    $indexdesc{"kde"} = "K Desktop Environment.";
    $indexdesc{"korean"} = "Ported software for the Korean market.";
    $indexdesc{"lang"} = "Computer languages.";
    $indexdesc{"languages"} = "Computer languages.";
    $indexdesc{"libraries"} = "Software development libraries.";
    $indexdesc{"mail"} = "Electronic mail packages and utilities.";
    $indexdesc{"math"} = "Mathematical computation software."; 
    $indexdesc{"mbone"} = "Applications and utilities for the mbone.";
    $indexdesc{"misc"} = "Miscellaneous utilities.";
    $indexdesc{"net"} = "Networking utilities."; 
    $indexdesc{"networking"} = "Networking utilities.";
    $indexdesc{"news"} = "USENET News support software.";
    $indexdesc{"numeric"} = "Mathematical computation software.";
    $indexdesc{"orphans"} = "Packages without a home elsewhere.";
    $indexdesc{"perl5"} = "Utilities/modules for the PERL5 language..";
    $indexdesc{"pilot"} = "Software for PalmPilot.";
    $indexdesc{"plan9"} = "Software from the plan9 Operating System.";
    $indexdesc{"print"} = "Utilities for dealing with printing.";
    $indexdesc{"programming"} = "Software development utilities and libraries.";
    $indexdesc{"russian"} = "Ported software for the Russian market.";
    $indexdesc{"security"} = "System security software.";
    $indexdesc{"shells"} = "Various shells (tcsh, bash, etc).";
    $indexdesc{"sysutils"} = "Various system utilities.";
    $indexdesc{"textproc"} = "Text processing/search utilities.";
    $indexdesc{"tcl75"} = "TCL v7.5 and packages which depend on it.";
    $indexdesc{"tcl76"} = "TCL v7.6 and packages which depend on it.";
    $indexdesc{"tcl80"} = "TCL v8.0 and packages which depend on it.";
    $indexdesc{"tk41"} = "Tk4.1 and packages which depend on it."; 
    $indexdesc{"tk42"} = "Tk4.2 and packages which depend on it.";
    $indexdesc{"tk80"} = "Tk8.0 and packages which depend on it.";
    $indexdesc{"troff"} = "TROFF Text formatting utilities.";
    $indexdesc{"util"} = "Various user utilities.";
    $indexdesc{"vietnamese"} = "Ported software for the Vietnamese market.";
    $indexdesc{"www"} = "WEB utilities (browers, HTTP servers, etc).";
    $indexdesc{"x11"} = "X Window System based utilities.";

    return (%indexdesc);
}


#
# return keyword aliases
#
sub get_INDEX_alias {

    my %indexalias = ();

    $indexalias{'development'} = 'devel';
    $indexalias{'emulation'} = 'emulat';
    $indexalias{'emulators'} = 'emulat';
    $indexalias{'utils'} = 'util';
    $indexalias{'utilities'} = 'util';
    $indexalias{'printing'} = 'print';

    return (%indexalias);
}


#
# return nice keyword descriptions
#
sub get_INDEX_keyword {

    my %indexkeyword = ();

    $indexkeyword{'devel'} = 'development';
    $indexkeyword{'print'} = 'printing';
    $indexkeyword{'emulat'} = 'emulators';
    $indexkeyword{'util'} = 'utilities';

    return (%indexkeyword);
}


#
# list of known ftp servers which serve packages
#
sub get_package_ftp_servers {

    my %ftpserver = ();

    $ftpserver{"Primary Site"}{'server'} = "ftp.freebsd.org";
    $ftpserver{"Primary Site"}{'URL'} = "ftp://ftp.freebsd.org/pub/FreeBSD/";
    $ftpserver{"3.0 SNAP Server"}{'server'} = "current.freebsd.org";
    $ftpserver{"3.0 SNAP Server"}{'URL'} = "ftp://current.freebsd.org/pub/FreeBSD/";
    $ftpserver{"2.2 SNAP Server"}{'server'} = "releng22.freebsd.org";
    $ftpserver{"2.2 SNAP Server"}{'URL'} = "ftp://releng22.freebsd.org/pub/FreeBSD/";
    $ftpserver{"2.1 SNAP Server"}{'server'} = "releng210.freebsd.org";
    $ftpserver{"2.1 SNAP Server"}{'URL'} = "ftp://releng210.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Argentina"}{'server'} = "ftp.ar.freebsd.org";
    $ftpserver{"Argentina"}{'URL'} = "ftp://ftp.ar.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Australia"}{'server'} = "ftp.au.freebsd.org";
    $ftpserver{"Australia"}{'URL'} = "ftp://ftp.au.freebsd.org/pub/FreeBSD/";
    $ftpserver{'Australia \#2'}{'server'} = 'ftp2.au.freebsd.org';
    $ftpserver{"Australia \#2"}{'URL'} = "ftp://ftp2.au.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Australia <#3"}{'server'} = "ftp3.au.freebsd.org";
    $ftpserver{"Australia <#3"}{'URL'} = "ftp://ftp3.au.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Australia \#4"}{'server'} = "ftp4.au.freebsd.org";
    $ftpserver{"Australia \#4"}{'URL'} = "ftp://ftp4.au.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Australia \#5"}{'server'} = "ftp5.au.freebsd.org";
    $ftpserver{"Australia \#5"}{'URL'} = "ftp://ftp5.au.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Brazil"}{'server'} = "ftp.br.freebsd.org";
    $ftpserver{"Brazil"}{'URL'} = "ftp://ftp.br.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Brazil \#2"}{'server'} = "ftp2.br.freebsd.org";
    $ftpserver{"Brazil \#2"}{'URL'} = "ftp://ftp2.br.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Brazil \#3"}{'server'} = "ftp3.br.freebsd.org";
    $ftpserver{"Brazil \#3"}{'URL'} = "ftp://ftp3.br.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Brazil \#4"}{'server'} = "ftp4.br.freebsd.org";
    $ftpserver{"Brazil \#4"}{'URL'} = "ftp://ftp4.br.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Brazil \#5"}{'server'} = "ftp5.br.freebsd.org";
    $ftpserver{"Brazil \#5"}{'URL'} = "ftp://ftp5.br.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Brazil \#6"}{'server'} = "ftp6.br.freebsd.org";
    $ftpserver{"Brazil \#6"}{'URL'} = "ftp://ftp6.br.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Brazil \#7"}{'server'} = "ftp7.br.freebsd.org";
    $ftpserver{"Brazil \#7"}{'URL'} = "ftp://ftp7.br.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Canada"}{'server'} = "ftp.ca.freebsd.org";
    $ftpserver{"Canada"}{'URL'} = "ftp://ftp.ca.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Czech Republic"}{'server'} = "ftp.cz.freebsd.org";
    $ftpserver{"Czech Republic"}{'URL'} = "ftp://ftp.cz.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Estonia"}{'server'} = "ftp.ee.freebsd.org";
    $ftpserver{"Estonia"}{'URL'} = "ftp://ftp.ee.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Finland"}{'server'} = "ftp.fi.freebsd.org";
    $ftpserver{"Finland"}{'URL'} = "ftp://ftp.fi.freebsd.org/pub/FreeBSD/";
    $ftpserver{"France"}{'server'} = "ftp.fr.freebsd.org";
    $ftpserver{"France"}{'URL'} = "ftp://ftp.fr.freebsd.org/pub/FreeBSD/";
    $ftpserver{"France \#2"}{'server'} = "ftp2.fr.freebsd.org";
    $ftpserver{"France \#2"}{'URL'} = "ftp://ftp2.fr.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Germany"}{'server'} = "ftp.de.freebsd.org";
    $ftpserver{"Germany"}{'URL'} = "ftp://ftp.de.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Germany \#2"}{'server'} = "ftp2.de.freebsd.org";
    $ftpserver{"Germany \#2"}{'URL'} = "ftp://ftp2.de.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Germany \#3"}{'server'} = "ftp3.de.freebsd.org";
    $ftpserver{"Germany \#3"}{'URL'} = "ftp://ftp3.de.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Germany \#4"}{'server'} = "ftp4.de.freebsd.org";
    $ftpserver{"Germany \#4"}{'URL'} = "ftp://ftp4.de.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Germany \#5"}{'server'} = "ftp5.de.freebsd.org";
    $ftpserver{"Germany \#5"}{'URL'} = "ftp://ftp5.de.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Germany \#6"}{'server'} = "ftp6.de.freebsd.org";
    $ftpserver{"Germany \#6"}{'URL'} = "ftp://ftp6.de.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Germany \#7"}{'server'} = "ftp7.de.freebsd.org";
    $ftpserver{"Germany \#7"}{'URL'} = "ftp://ftp7.de.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Holland"}{'server'} = "ftp.nl.freebsd.org";
    $ftpserver{"Holland"}{'URL'} = "ftp://ftp.nl.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Hong Kong"}{'server'} = "ftp.hk.super.net";
    $ftpserver{"Hong Kong"}{'URL'} = "ftp://ftp.hk.super.net/pub/FreeBSD/";
    $ftpserver{"Iceland"}{'server'} = "ftp.is.freebsd.org";
    $ftpserver{"Iceland"}{'URL'} = "ftp://ftp.is.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Ireland"}{'server'} = "ftp.ie.freebsd.org";
    $ftpserver{"Ireland"}{'URL'} = "ftp://ftp.ie.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Israel"}{'server'} = "ftp.il.freebsd.org";
    $ftpserver{"Israel"}{'URL'} = "ftp://ftp.il.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Israel \#2"}{'server'} = "ftp2.il.freebsd.org";
    $ftpserver{"Israel \#2"}{'URL'} = "ftp://ftp2.il.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Japan"}{'server'} = "ftp.jp.freebsd.org";
    $ftpserver{"Japan"}{'URL'} = "ftp://ftp.jp.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Japan \#2"}{'server'} = "ftp2.jp.freebsd.org";
    $ftpserver{"Japan \#2"}{'URL'} = "ftp://ftp2.jp.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Japan \#3"}{'server'} = "ftp3.jp.freebsd.org";
    $ftpserver{"Japan \#3"}{'URL'} = "ftp://ftp3.jp.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Japan \#4"}{'server'} = "ftp4.jp.freebsd.org";
    $ftpserver{"Japan \#4"}{'URL'} = "ftp://ftp4.jp.freebsd.org/pub/FreeBSD/";    
    $ftpserver{"Japan \#5"}{'server'} = "ftp5.jp.freebsd.org";    
    $ftpserver{"Japan \#5"}{'URL'} = "ftp://ftp5.jp.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Japan \#6"}{'server'} = "ftp6.jp.freebsd.org";
    $ftpserver{"Japan \#6"}{'URL'} = "ftp://ftp6.jp.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Korea"}{'server'} = "ftp.kr.freebsd.org";
    $ftpserver{"Korea"}{'URL'} = "ftp://ftp.kr.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Korea \#2"}{'server'} = "ftp2.kr.freebsd.org";
    $ftpserver{"Korea \#2"}{'URL'} = "ftp://ftp2.kr.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Poland"}{'server'} = "ftp.pl.freebsd.org";
    $ftpserver{"Poland"}{'URL'} = "ftp://ftp.pl.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Portugal"}{'server'} = "ftp.pt.freebsd.org";
    $ftpserver{"Portugal"}{'URL'} = "ftp://ftp.pt.freebsd.org/pub/misc/FreeBSD/";
    $ftpserver{"Portugal \#2"}{'server'} = "ftp2.pt.freebsd.org";
    $ftpserver{"Portugal \#2"}{'URL'} = "ftp://ftp2.pt.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Russia"}{'server'} = "ftp.ru.freebsd.org";
    $ftpserver{"Russia"}{'URL'} = "ftp://ftp.ru.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Russia \#2"}{'server'} = "ftp2.ru.freebsd.org";
    $ftpserver{"Russia \#2"}{'URL'} = "ftp://ftp2.ru.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Russia \#3"}{'server'} = "ftp3.ru.freebsd.org";
    $ftpserver{"Russia \#3"}{'URL'} = "ftp://ftp3.ru.freebsd.org/pub/FreeBSD/";
    $ftpserver{"South Africa"}{'server'} = "ftp.za.freebsd.org";
    $ftpserver{"South Africa"}{'URL'} = "ftp://ftp.za.freebsd.org/pub/FreeBSD/";
    $ftpserver{"South Africa \#2"}{'server'} = "ftp2.za.freebsd.org";
    $ftpserver{"South Africa \#2"}{'URL'} = "ftp://ftp2.za.freebsd.org/pub/FreeBSD/";
    $ftpserver{"South Africa \#3"}{'server'} = "ftp3.za.freebsd.org";    
    $ftpserver{"South Africa \#3"}{'URL'} = "ftp://ftp3.za.freebsd.org/pub/FreeBSD/";
    $ftpserver{"South Africa \#4"}{'server'} = "ftp4.za.freebsd.org";
    $ftpserver{"South Africa \#4"}{'URL'} = "ftp://ftp4.za.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Sweden"}{'server'} = "ftp.se.freebsd.org";
    $ftpserver{"Sweden"}{'URL'} = "ftp://ftp.se.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Sweden \#2"}{'server'} = "ftp2.se.freebsd.org";
    $ftpserver{"Sweden \#2"}{'URL'} = "ftp://ftp2.se.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Sweden \#3"}{'server'} = "ftp3.se.freebsd.org";
    $ftpserver{"Sweden \#3"}{'URL'} = "ftp://ftp3.se.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Taiwan"}{'server'} = "ftp.tw.freebsd.org";
    $ftpserver{"Taiwan"}{'URL'} = "ftp://ftp.tw.freebsd.org/pub/FreeBSD";
    $ftpserver{"Taiwan \#2"}{'server'} = "ftp2.tw.freebsd.org";
    $ftpserver{"Taiwan \#2"}{'URL'} = "ftp://ftp2.tw.freebsd.org/pub/FreeBSD";
    $ftpserver{"Taiwan \#3"}{'server'} = "ftp3.tw.freebsd.org";
    $ftpserver{"Taiwan \#3"}{'URL'} = "ftp://ftp3.tw.freebsd.org/pub/FreeBSD/";
    $ftpserver{"Thailand"}{'server'} = "ftp.nectec.or.th";
    $ftpserver{"Thailand"}{'URL'} = "ftp://ftp.nectec.or.th/pub/mirrors/FreeBSD/";
    $ftpserver{"UK"}{'server'} = "ftp.uk.freebsd.org";
    $ftpserver{"UK"}{'URL'} = "ftp://ftp.uk.freebsd.org/pub/FreeBSD/";
    $ftpserver{"UK \#2"}{'server'} = "ftp2.uk.freebsd.org";
    $ftpserver{"UK \#2"}{'URL'} = "ftp://ftp2.uk.freebsd.org/pub/FreeBSD/";
    $ftpserver{"UK \#3"}{'server'} = "ftp3.uk.freebsd.org";
    $ftpserver{"UK \#3"}{'URL'} = "ftp://ftp3.uk.freebsd.org/pub/FreeBSD/";
    $ftpserver{"UK \#4"}{'server'} = "ftp4.uk.freebsd.org";
    $ftpserver{"UK \#4"}{'URL'} = "ftp://ftp4.uk.freebsd.org/pub/FreeBSD/";
    $ftpserver{"USA"}{'server'} = "ftp.freebsd.org";
    $ftpserver{"USA"}{'URL'} = "ftp://ftp.freebsd.org/pub/FreeBSD/";
    $ftpserver{"USA \#2"}{'server'} = "ftp2.freebsd.org";
    $ftpserver{"USA \#2"}{'URL'} = "ftp://ftp2.freebsd.org/pub/FreeBSD/";
    $ftpserver{"USA \#3"}{'server'} = "ftp3.freebsd.org";
    $ftpserver{"USA \#3"}{'URL'} = "ftp://ftp3.freebsd.org/pub/FreeBSD/";
    $ftpserver{"USA \#4"}{'server'} = "ftp4.freebsd.org";
    $ftpserver{"USA \#4"}{'URL'} = "ftp://ftp4.freebsd.org/pub/FreeBSD/";
    $ftpserver{"USA \#5"}{'server'} = "ftp5.freebsd.org";
    $ftpserver{"USA \#5"}{'URL'} = "ftp://ftp5.freebsd.org/pub/FreeBSD/";
    $ftpserver{"USA \#6"}{'server'} = "ftp6.freebsd.org";
    $ftpserver{"USA \#6"}{'URL'} = "ftp://ftp6.freebsd.org/pub/FreeBSD/";

    return %ftpserver;

}
