#
# access to apache configuration files. Supports both the unenlightened and
# englightened webmaster
#
# Note: currently only supports adding aliases
#
# Created: 1998-02-14 by Berend
#

# Copyright (c) 1998
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 


package access_apache;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_httpdconf_location get_httpdconf_directives get_srmconf_location get_srmconf_directives add_alias);


use Sys::Hostname;


# read and parse apache config files
# Parameters: 
#   input $filename
#   output %directives
#

sub get_directives {

    my($filename) = @_;
    my %directives = ();

    open (FILE, $filename);

    while ($Line = <FILE>) {
	chop($Line);
	$Line =~ s/^\s+//;         # trim spaces

	#distinguish between group/nongroup directives
	if ($Line =~ /^\</) { #group
	    @fields = split /\s+/, $Line, 2;	    
	    $fields[0] =~ s/^\<//;
	    $fields[1] =~ s/\>$//;
	    $directives{$fields[0]}{'value'} = $fields[1];
	    $end = "</$fields[0]>";
	    while (($Line !~ /$end/i) and ($Line = <FILE>)) {}
	}
	else { #single line directive
	    @fields = split /\s+/, $Line, 2;
	    $directives{$fields[0]}{'value'} = $fields[1];
	}
    }

    close (FILE);

    return %directives;

}



#
# where can we find httpd.conf?
#

sub get_httpdconf_location {

    if (-e '/usr/local/etc/apache/httpd.conf') {
	return '/usr/local/etc/apache/httpd.conf';
    }
    elsif (-e '/usr/local/www/server/httpd.conf') {
	return '/usr/local/www/server/httpd.conf';
    }
    else {
	return '';
    }
}



# read and parse everything in httpd.conf
# Parameters: 
#   input none
#   output %directives
#

sub get_httpdconf_directives {

    my %directives = ();
    my $filename = get_httpdconf_location;
    (%directives) = get_directives($filename);

}



#
# where can we find srm.conf?
#

sub get_srmconf_location {
    my %directives = ();
    (%directives) = get_httpdconf_directives;
    $ResourceConfig = $directives{'ResourceConfig'}{'value'};
    if ($ResourceConfig eq '') {
	$ResourceConfig = get_httpdconf_location;
	$ResourceConfig = `dirname $ResourceConfig`;
	chop($ResourceConfig);
	$ResourceConfig = "$ResourceConfig/srm.conf";
    }
    elsif ($ResourceConfig eq '/dev/null') {
	$ResourceConfig = get_httpdconf_location;
    }
    return $ResourceConfig;
}



# read and parse everything in srm.conf 
# (if exists, else just returns httpd.conf)
# Parameters: 
#   input none
#   output %directives
#

sub get_srmconf_directives {

    my %directives = ();
    my $filename = get_srmconf_location();
    (%directives) = get_directives($filename);

}



#
# add an alias to apache
# if the alias already exists, it is replaced
# This alias is also made a script alias for .pl files, and access is only
# allowed to the local machine
# input: $filename, $Alias, $Location
# output: none
#
# Note: current very very specific for FreEasy

sub add_alias {

    my($filename) = @_;
    $tmpfilename = "$filename.new";
    shift @_;
    my($Alias) = @_;
    shift @_;
    my($Location) = @_;
    my %directives = ();
    my $Hostname = hostname();
    my $AliasWritten = 0;

    open (FILE, $filename);
    open (NEWFILE, "> $tmpfilename");

    while ($Line = $OrgLine = <FILE>) {
	chop($Line);
	$Line =~ s/^\s+//;         # trim spaces

	#distinguish between group/nongroup directives
	if ($Line =~ /^\</) { #group
	    @fields = split /\s+/, $Line, 2;	    
	    $fields[0] =~ s/^\<\s*//;
	    $fields[1] =~ s/\s*\>$//;
	    if ( ($fields[0] =~ /Location/i) &&
		 ($fields[1] =~ /\/$Alias\//) ) {
		$end = "</Location>";
		while (($Line !~ /$end/i) and ($Line = <FILE>)) {};
	    }
	    else {
		print NEWFILE $OrgLine;
	    }
	}
	else { #single line directive
	    @fields = split /\s+/, $Line, 2;
	    if ( ($fields[0] =~ /ScriptAlias/i) &&
		    ($fields[1] =~ /\/$Alias\//) ) {
		#delete ScriptAlias (used in FreEasy v0.2)
	    }
	    elsif ( ($fields[0] =~ /Alias/i) &&
		 ($fields[1] =~ /\/$Alias\//) ) {
		print NEWFILE "Alias /$Alias/ $Location\n";
		print NEWFILE "<Location /$Alias/>\n";
		print NEWFILE "AddHandler cgi-script .pl\n";
		print NEWFILE "order deny,allow\n";
		print NEWFILE "deny from all\n";
		print NEWFILE "allow from $Hostname\n";
		print NEWFILE "</Location>\n";
		$AliasWritten = 1;
	    }
	    else {
		print NEWFILE $OrgLine;
	    }
	}
    }

    if ( ! $AliasWritten) {
	print NEWFILE "Alias /$Alias/ $Location\n";
	print NEWFILE "<Location /$Alias/>\n";
	print NEWFILE "AddHandler cgi-script .pl\n";
	print NEWFILE "order deny,allow\n";
	print NEWFILE "deny from all\n";
	print NEWFILE "allow from $Hostname\n";
	print NEWFILE "</Location>\n";
    }

    close (NEWFILE);
    close (FILE);

    #save the old $filename, and move $tmpfilename into its place
    rename($filename, "$filename.bak")   or die "can't rename $filename to .bak: $!";
    rename($tmpfilename, $filename) or die "can't rename $tmpfilename to $filename: $!";

    return %directives;

}
