#
# low level parse routine for termcap like files, see the manpage
# on termcap(5)
#
# Created: 1998-02-19 by Berend
#

# Copyright (c) 1998
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 


package access_termcap_file;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_all_entries);


#
# read a given termcap file
# and return them all with their individual fields
# Parameters: 
#   input $filename, 
#   output %entries
#
sub get_all_entries {

    my($filename) = @_;
    my(%entries) = ();
    my(@capabilities) = ();
    my $capability = '';
    my(@names) = ();

    open (FILE, $filename) or die "Can't open $filename";

    while ($Line = <FILE>) {

	#remove newline and spaces before and after this line
	chop($Line);
	$Line =~ s/^\s+//;
	$Line =~ s/\s+$//;

	#if it is not a comment or empty line
	if ( ($Line !~ /^\#/) && ($Line ne '') ) {

	    #we have a, possibly multi-line, entry
	    while ($Line =~ /\\$/) {
		chop($Line);
		$NextLine = <FILE>;
		$NextLine =~ s/^\s+//;
		$NextLine =~ s/\s+$//;
		$Line .= $NextLine;
	    }

	    #remove empty fields
	    $Line =~ s/\:\:/\:/g;

	    #entire line read, split up its fields
	    #and get it into a hash
	    @capabilities = split /:/, $Line;
	    @names = split /\|/, $capabilities[0];
	    shift @capabilities;
	    $entries{$names[0]}{'longname'} = $names[scalar(@names)-1];
	    foreach $capability (@capabilities) {

		# if capability is not commented out
		if ($capability !~ /^\./) {

		    #distinguish between capability types
		    if ($capability =~ /..\=/) { #string cap
			@fields = split /\=/, $capability;
		    }
		    elsif ($capability =~ /..\#/) { #numeric cap
			@fields = split /\#/, $capability;
		    }
		    else { #boolean cap
			$fields[0] = $capability;
			$fields[1] = 'true';
		    }
		    # add the individual capability
		    $entries{$names[0]}{$fields[0]} = $fields[1];
		}
	    }

	}

    }

    close(FILE);

    return %entries;

}
