#
# provides access to /etc/ttys
#
# Created: 1998-02-05 by Berend
#

# Copyright (c) 1998
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

package access_ttys;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(ttys_get_all_entries get_all_console_types change_single_user_security change_virtual_consoles);


#
# get every entry in /etc/ttys
# input: none
# output: %ttys


sub ttys_get_all_entries {

    my @users = ();
    my @fields = ();
    my $i = 0;
    my %ttys = ();

    open (FILE, '/etc/ttys');

    while ($Line = <FILE>) {
	chop($Line);
	$Line =~ s/^\s+//;         # trim spaces
	@fields = split /#/, $Line, 2;
	$Line = $fields[0];
	if ( ($Line ne '') &&
	     ($Line !~ /^\#/) ) { 
	    (@fields) = ($Line =~ /(".*?"|[^\s][^\s]*)/g);
	    $ttys{$fields[0]}{'command'} = $fields[1];
	    $ttys{$fields[0]}{'type'} = $fields[2];
	    for ($i = 2; $i < scalar @fields; $i++) {
		if ($fields[$i] =~ /(^on$|^off$)/i) {
		    $ttys{$fields[0]}{'on'} = $fields[$i];		    
		}
		elsif ($fields[$i] =~ /(^secure$|^insecure$)/i) {
		    $ttys{$fields[0]}{'secure'} = $fields[$i];		    
		}
		elsif ($fields[$i] =~ /^dialin$/i) {
		    $ttys{$fields[0]}{'dialin'} = $fields[$i];		    
		}
		elsif ($fields[$i] =~ /^network$/i) {
		    $ttys{$fields[0]}{'network'} = $fields[$i];		    
		}
	        else {
		    $ttys{$fields[0]}{'group'} = $fields[$i];		    
	        }
	    }
            # ignore window, group and comment for now
	}
    }

    close (FILE);

    return %ttys;

}


#
# return every valid terminal type for a FreeBSD console
# input: none
# output: @terminaltypes

sub get_all_console_types {

    my %termtypes = ();
    $termtypes{'cons25w'} = 'console 80x25 raw';
    $termtypes{'cons25'} = 'console 80x25';
    $termtypes{'cons25-m'} = 'console 80x25 mono';
    $termtypes{'cons30'} = 'console 80x30';
    $termtypes{'cons30-m'} = 'console 80x30 mono';
    $termtypes{'cons43'} = 'console 80x43';
    $termtypes{'cons43-m'} = 'console 80x43 mono';
    $termtypes{'cons50'} = 'console 80x50';
    $termtypes{'cons50-m'} = 'console 80x50 mono';
    $termtypes{'cons60'} = 'console 80x60';
    $termtypes{'cons60-m'} = 'console 80x60 mono';
    $termtypes{'cons25r'} = 'console 80x25 russian code table';
    $termtypes{'cons25r-m'} = 'console 80x25 russian code table mono';
    $termtypes{'cons50r'} = 'console 80x50 russian code table';
    $termtypes{'cons50r-m'} = 'console 80x50 russian code table mono';
    $termtypes{'cons25l1'} = 'console 80x25 ISO 8859-1 FreeBSD';
    $termtypes{'cons25l1-m'} = 'console 80x25 ISO 8859-1 FreeBSD mono';
    $termtypes{'cons50l1'} = 'console 80x50 ISO 8859-1 FreeBSD';
    $termtypes{'cons50l1-m'} = 'console 80x50 ISO 8859-1 FreeBSD mono';

    return(%termtypes);
}


#
# create a temporary ttys file
#

sub create_temp_ttys_file {

    my $filename = '/etc/ttys';
    my $tmpfilename = "$filename.new";

    #create a temporary file to save changes in
    #make sure it has the same access rights as the original $filename
    open (FILE, $filename)              or die "can't open $filename: $!";
    ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
     $atime,$mtime,$ctime,$blksize,$blocks) = stat $filename;
    open (FILEOUT, "> $tmpfilename")    or die "can't create $tmpfilename: $!";
    flock FILEOUT, 2;
    chmod $mode, $tmpfilename;

    return ($filename, $tmpfilename);

}


# replace the original file, or, if not modified, remove the new file

sub replace_ttys_file {

    my($modified) = @_;
    shift @_;
    my($filename) = @_;
    shift @_;
    my($tmpfilename) = @_;

    if ( $modified ) {
	rename($filename, "$filename.bak")   or die "can't rename $filename to .bak: $!";
	rename($tmpfilename, $filename) or 
	    die "can't rename $tmpfilename to $filename: $!";
    }
    else {
	unlink $tmpfilename;
    }

}



#
# set secure/insecure of console entry in /etc/ttys
# input: $secure (secure/insecure)
# output: $modified

sub change_single_user_security {

    my($secure) = @_;
    my $modified = 0;
    my $i;

    ($filename, $tmpfilename) = create_temp_ttys_file;

    while ($OrgLine = <FILE>) {
	chop($OrgLine);
	$OrgLine =~ s/^\s+//;         # trim spaces
	@fields = split /#/, $OrgLine, 2;
	$Line = $fields[0];
	if ( ($Line ne '') &&
	     ($Line !~ /^\#/) ) { 
	    (@fields) = ($Line =~ /(".*?"|[^\s][^\s]*)/g);
	    if ($fields[0] eq 'console') {
		print FILEOUT 'console';
		for ($i = 1; $i < scalar @fields; $i++) {		
		    if ($fields[$i] =~ /(^secure$|^insecure$)/i) {
			print FILEOUT "\t$secure";
			if ($secure ne $fields[$i]) { $modified = 1; }
		    }
		    else {  print FILEOUT "\t$fields[$i]"; }
		}		    
		print FILEOUT "\n";
		while (<FILE>) { print FILEOUT "$_"; }
		last;
	    }
	    else { print FILEOUT "$OrgLine\n"; }
	}
        else { print FILEOUT "$OrgLine\n"; }
    }

    close (FILE);
    close (FILEOUT);

    replace_ttys_file($modified, $filename, $tmpfilename);

    return($modified);

}



#
# change virtual console setup
# input: %ttys 
#   (structure like ttys_get_all_entries but should contain only vty's)
# output: $modified

sub change_virtual_consoles {

    my(%ttys) = @_;
    my $modified = 0;
    my $i;
    my $vtyswritten = 0;

    ($filename, $tmpfilename) = create_temp_ttys_file;

    # you may not turn off the main console
    $ttys{'ttyv0'}{'on'} = 'on';

    while ($OrgLine = <FILE>) {
	chop($OrgLine);
	$OrgLine =~ s/^\s+//;         # trim spaces
	@fields = split /#/, $OrgLine, 2;
	$Line = $fields[0];
	if ( ($Line ne '') &&
	     ($Line !~ /^\#/) ) { 
	    (@fields) = ($Line =~ /(".*?"|[^\s][^\s]*)/g);
	    if (exists $ttys{$fields[0]}) {
		print FILEOUT "$fields[0]\t$fields[1]\t$ttys{$fields[0]}{'type'}";
		if ($ttys{$fields[0]}{'type'} ne $fields[2]) { 
		    $modified = 1; 
		}
		for ($i = 3; $i < scalar @fields; $i++) {		
		    if ($fields[$i] =~ /(^on$|^off$)/i) {
			print FILEOUT "\t$ttys{$fields[0]}{'on'}";
			if ($ttys{$fields[0]}{'on'} ne $fields[$i]) { 
			    $modified = 1; 
			}
		    }
		    elsif ($fields[$i] =~ /(^secure$|^insecure$)/i) {
			print FILEOUT "\t$ttys{$fields[0]}{'secure'}";
			if ($ttys{$fields[0]}{'secure'} ne $fields[$i]) { 
			    $modified = 1; 
			}
		    }
		    else {  print FILEOUT "\t$fields[$i]"; }
		}
		delete $ttys{$fields[0]};
		print FILEOUT "\n";
		$vtyswritten = 1;
	    }
	    else { 
		# print remaining ttys
		if ($vtyswritten) {
		    foreach $tty (sort keys %ttys) {
			print FILEOUT "$tty\t\"/usr/libexec/getty Pc\"\t$ttys{$tty}{'type'}\t$ttys{$tty}{'on'}\t$ttys{$tty}{'secure'}\n";
		    }
		    $vtyswritten = 0;
		}
		if ($fields[0] !~ /^ttyv/) { print FILEOUT "$OrgLine\n"; }
	    }
	}
        else { 
	    if ($OrgLine !~ /irtual/) {
		if ($vtyswritten) {
		    # print remaining ttys
		    foreach $tty (sort keys %ttys) {
			print FILEOUT "$tty\t\"/usr/libexec/getty Pc\"\t$ttys{$tty}{'type'}\t$ttys{$tty}{'on'}\t$ttys{$tty}{'secure'}\n";
		    }
		    $vtyswritten = 0;
		}
	    }
	    print FILEOUT "$OrgLine\n"; 	    
	}
    }

    close (FILE);
    close (FILEOUT);


    replace_ttys_file($modified, $filename, $tmpfilename);

    return ($modified);
}
