#
# low level parse routine for rc.conf like files, i.e. files which look like
#   <var> = <some value>
# comments in this file start with a '#' and are skipped.
#
# Created: 1998-01-14 by Berend
#

# Copyright (c) 1998
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 


package access_var_file;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_all_vars get_some_vars change_vars);


#
# read a given file with variables
# and return them all with their contents
# Parameters: 
#   input $filename, 
#   output %vars
#
sub get_all_vars {

    my($filename) = @_;
    my %vars = ();

    open (FILE, $filename);

    while ($Line = <FILE>) {
	chop($Line);
	@fields = split /#/, $Line, 2;
	@fields = split /=/, $fields[0];
	if ( $fields[1] ne '' ) {
	    $fields[0] =~ s/^\s+//;         # trim spaces
	    $fields[0] =~ s/\s+$//;
	    $fields[1] =~ s/^\s*\"\s*//;
	    $fields[1] =~ s/\s*\"\s+$//;
	    $vars{$fields[0]} = $fields[1];
	}
    }

    close (FILE);

    return %vars;

}


#
# given variable names, returns them with their contents
# from the given var file (if found, else contents is empty)
# Parameters: 
#   input $filename, $var1, $var2, ...
#   output %vars

sub get_some_vars {

    my($filename) = @_[0];
    my %allvars = ();
    my %vars = ();

    (%allvars) = get_all_vars($filename);
    for ($i = 1; $i < scalar(@_); $i++) {

	if (exists($allvars{@_[$i]})) {
	    $vars{@_[$i]} = $allvars{@_[$i]};
	}
	else {
	    $vars{@_[$i]} = '';
	}

    }

    return %vars;
    
}


#
# given an hash array of variables and their contents, modify
# the variables in the given file. If the variable does not exist
# in the original file, it is appended at the end.
# Parameters: 
#   input $filename, %vars
#

sub change_vars {

    my($filename) = @_;
    my($tmpfilename) = $filename;
    $tmpfilename = "$filename.new";
    shift @_;
    my %vars = @_;
    my %oldvars = ();
    my $modified = 0;

    #create a temporary file to save changes in
    #make sure it has the same access rights as the original $filename
    open (FILEIN, $filename)            or die "can't open $filename: $!";
    ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
     $atime,$mtime,$ctime,$blksize,$blocks) = stat $filename;
    open (FILEOUT, "> $tmpfilename")    or die "can't create $tmpfilename: $!";
    flock FILEOUT, 2;
    chmod $mode, $tmpfilename;

    #loop through $filename and copy the (modified) lines to $tmpfilename
    while ($Line = <FILEIN>) {
	chop($Line);
	@fields = split /#/, $Line, 2;
	$comment = $fields[1];
	@fields = split /=/, $fields[0];
	if ( $fields[1] ne '' ) {
	    $fields[0] =~ s/^\s+//;         # trim spaces
	    $fields[0] =~ s/\s+$//;
	    $fields[1] =~ s/^\s*\"//;
	    $fields[1] =~ s/\"\s+$//;
	    $oldvars{$fields[0]} = $fields[1];
	    if (exists($vars{$fields[0]})) {
		if ($vars{$fields[0]} ne $fields[1])  {
		    $varassign = "$fields[0]=\"$vars{$fields[0]}\"";
		    if (($comment ne '') && 
			(length $varassign < 16)) {
			$varassign = "$varassign\t";
		    }
		    if ($comment ne '' ) {
			$comment = "\t#$comment";
		    }
		    print FILEOUT "$varassign$comment\n";
		    delete $vars{$fields[0]};
		    $modified = 1;
		}
		else {
		    print FILEOUT "$Line\n";
		    delete $vars{$fields[0]};
		}
	    }
	    else {
		print FILEOUT "$Line\n";
	    }
	}
	else {
	    print FILEOUT "$Line\n";
	}
    }

    # for every var not in filename, append it at the end
    foreach $var (keys %vars) {
	print FILEOUT "$var=\"$vars{$var}\"\n";
	$modified = 1;
    }

    close (FILEIN);
    close (FILEOUT);

    if ( $modified ) {

	#save the old $filename, and move $tmpfilename into its place
	rename($filename, "$filename.bak")   or die "can't rename $filename to .bak: $!";
	rename($tmpfilename, $filename) or 
	    system("mv", $tmpfilename, $filename) or
	    die "can't rename $tmpfilename to $filename: $!";

    }
    else {
	unlink $tmpfilename;
    }

    return ($modified, %oldvars);
}


1;
# access_var_file.pm

