package changekernel;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(change_ident_kernel change_cpu_kernel change_network_card_kernel change_audio_card_kernel change_filesystems_kernel);

use removeleadingspace;

$KernelPath = "/usr/src/sys/i386/conf";

#
# Change the indentification name of the kernel
# (C) 1997 by Wouter de Boer
#
# Change the CPU of the kernel
# (C) 1997 by Wouter de Boer
#
# Change the network card of the kernel
# (C) 1997 by Wouter de Boer
#
# Change the audio card of the kernel
# (C) 1997 by Wouter de Boer
#
# Usage:
#    change_ident_kernel arg1
#
#	arg1: Kernel Name
#
#    change_cpu_kernel arg1, arg2
#
#	arg1: CPU type
#	arg2: Kernel Name
#
#    change_network_card_kernel arg1, arg2, arg3, arg4, arg5
#       arg6, arg7
#
#	arg1: Kernel Name
#	arg2: Selected Network Cards
#	arg3: Disabled Network Cards
#	arg4: Enabled Port
#	arg5: Enabled IRQ
#       arg6: fddi support (Yes/No)
#       arg7: sppp support (Yes/No)
#
#    change_audio_card_kernel arg1, arg2, arg3, arg4, arg5,
#	arg6, arg7, arg8
#
#	arg1: Kernel Name
#	arg2: Size Enabled Cards
#	arg3: Total Audio Cards
#	arg4: All Audio Cards
#	arg5: Selected Audio Cards
#	arg6: Enabled Port
#	arg7: Enabled IRQ
#	arg8: Enabled DMA
#
#    change_filesystems_kernel arg1, arg2, arg3, arg4, arg5
#
#       arg1: Kernel Name
#       arg2: Size Enabled Filesystems
#       arg3: Total FileSystems
#       arg4: All FileSystems
#       arg5: Enabled FileSystems
#

sub change_ident_kernel {

my($KernelName) = $_[0];

open (FILE, "$KernelPath/$KernelName");
open (FILE1, "> $KernelPath/new$KernelName");

while ($Line = <FILE>) {

  if ($Line =~ /^ident/) {

    $Line =~ s/GENERIC/$KernelName/;
    print FILE1 "$Line";
  } else {

      print FILE1 "$Line";
    }
}

#
# close both files
#

close(FILE);
close(FILE1);

#
# Copy the changed one to the old one and remove the temp one
#

system ("cp $KernelPath/new$KernelName $KernelPath/$KernelName");

system ("rm $KernelPath/new$KernelName");

}

sub change_cpu_kernel {

my($CPU, $KernelName) = @_;

open (FILE, "$KernelPath/$KernelName");
open (FILE1, "> $KernelPath/new$KernelName");

while ($Line = <FILE>) {

  while (($Line =~ /^cpu/) || ($Line =~ /^#cpu/)) {

    $Line = <FILE>;
  }

  if ($Line =~ /^machine/) {

    print FILE1 "$Line";

    if ($CPU eq "i386") {

      print FILE1 "cpu\t\t\"I386_CPU\"\n";
      print FILE1 "#cpu\t\t\"I486_CPU\"\n";
      print FILE1 "#cpu\t\t\"I586_CPU\"\n";
      print FILE1 "#cpu\t\t\"I686_CPU\"\n";

    } elsif ($CPU eq "i486") {

	print FILE1 "#cpu\t\t\"I386_CPU\"\n";
	print FILE1 "cpu\t\t\"I486_CPU\"\n";
	print FILE1 "#cpu\t\t\"I586_CPU\"\n";
	print FILE1 "#cpu\t\t\"I686_CPU\"\n";

      } elsif ($CPU eq "i586") {

	  print FILE1 "#cpu\t\t\"I386_CPU\"\n";
	  print FILE1 "#cpu\t\t\"I486_CPU\"\n";
	  print FILE1 "cpu\t\t\"I586_CPU\"\n";
	  print FILE1 "#cpu\t\t\"I686_CPU\"\n";

	} elsif ($CPU eq "i686") {

	    print FILE1 "#cpu\t\t\"I386_CPU\"\n";
	    print FILE1 "#cpu\t\t\"I486_CPU\"\n";
	    print FILE1 "#cpu\t\t\"I586_CPU\"\n";
	    print FILE1 "cpu\t\t\"I686_CPU\"\n";

	  }
  } else {

      print FILE1 "$Line";
    }
}

#
# close both files
#

close(FILE);
close(FILE1);

#
# Copy the changed one to the old one and remove the temp one
#

system ("cp $KernelPath/new$KernelName $KernelPath/$KernelName");

system ("rm $KernelPath/new$KernelName");

}

sub change_network_card_kernel {

my($KernelName, $EnabledCards, $DisabledCards, $EnabledPort, $EnabledIRQ, $FDDI, $SPPP) = @_;

#
# Open both files
#

open (FILE, "$KernelPath/$KernelName");
open (FILE1, "> $KernelPath/new$KernelName");

#
# Obtain size of the both array
#

$EnabledSize = scalar @{$EnabledCards};
$DisabledSize = scalar @{$DisabledCards};

while ($Line = <FILE>) {

  chop($Line);

  $Line = remove_leading_space $Line;

  if (($Line =~ /^pseudo-device/) && ($Line =~ /loop/)) {

    print FILE1 "$Line\n";

    $Line = <FILE>;

    chop($Line);

    $Line = remove_leading_space $Line;

    if ($FDDI eq "Yes") {

      print FILE1 "pseudo-device\tfddi\t\t\#Generic FDDI\n";
    }

    if ($SPPP eq "Yes") {

      print FILE1 "pseudo-device\tsppp\t\t\#Generic Synchronous SPPP\n";
    }
  }

  if (($Line =~ /^device/) || ($Line =~ /^\#device/)) {

    $Enabled = $Disabled = 1;

    for ($EnabledCounter = 0; $EnabledCounter < $EnabledSize; $EnabledCounter++) {

       if ($Line =~ /@{$EnabledCards}[$EnabledCounter]/) {

	 $Line =~ s/^#//;
	 @TempArray = split (/\s+/, $Line);

	 $Size = scalar @TempArray;
	 $TempArray[5] = @{$EnabledPort}[$EnabledCounter];
	 $TempArray[8] = @{$EnabledIRQ}[$EnabledCounter];
	
	 for ($Counter = 0; $Counter < $Size; $Counter++) {

	    print FILE1 "$TempArray[$Counter] ";
	 }

	 print FILE1 "\n";
	 $LastKnownEnabledCard = @{$EnabledCards}[$EnabledCounter];
	 splice (@{$EnabledCards}, $EnabledCounter, 1);
	 splice (@{$EnabledPort}, $EnabledCounter, 1);
	 splice (@{$EnabledIRQ}, $EnabledCounter, 1);
	 $EnabledSize = scalar @{$EnabledCards};
	 $Enabled = 0;
       }
    }

    for ($DisabledCounter = 0; $DisabledCounter < $DisabledSize; $DisabledCounter++) {

       if ($Line =~ /@{$DisabledCards}[$DisabledCounter]/) {

	 $Line =~ s/^#//;
	 print FILE1 "#$Line\n";
	 $Disabled = 0;
       }

    }

    if (($Enabled) && ($Disabled)) {

      print FILE1 "$Line\n";
    }

  } else {

      print FILE1 "$Line\n";
    }
}

#
# close both files
#

close (FILE);
close (FILE1);

#
# Copy the changed one to the old one and remove the temp one
#

system ("cp $KernelPath/new$KernelName $KernelPath/$KernelName");

system ("rm $KernelPath/new$KernelName");

#
# Write the cards which are not in the kernel file to the kernel
# file
#

if (length($LastKnownEnabledCard) == 0) {

  $LastKnownEnabledCard = "ep0";
}

if ($EnabledSize > 0) {

  #
  # Open both files
  #

  open (FILE, "$KernelPath/$KernelName");
  open (FILE1, "> $KernelPath/new$KernelName");

  while ($Line = <FILE>) {

    chop($Line);

    $Line = remove_leading_space $Line;

    if (($Line =~ /^device/) || ($Line =~ /^#device/)) {

      if ($Line =~ /$LastKnownEnabledCard/) {

	print FILE1 "$Line\n";

        for ($EnabledCounter = 0; $EnabledCounter < $EnabledSize; $EnabledCounter++) {
	   if (@{$EnabledCards}[$EnabledCounter] eq "ar0") {
	      
	     print FILE1 "device ar0 at isa? port @{$EnabledPort}[$EnabledCounter] net irq @{$EnabledIRQ}[$EnabledCounter] iomem 0xd0000 vector arintr\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "cx0") {

	     print FILE1 "device cx0 at isa? port @{$EnabledPort}[$EnabledCounter] net irq @{$EnabledIRQ}[$EnabledCounter] drq 7 vector cxintr\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "eg0") {

	     print FILE1 "device eg0 at isa? port @{$EnabledPort}[$EnabledCounter] net irq @{$EnabledIRQ}[$EnabledCounter] vector egintr\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "el0") {

	     print FILE1 "device el0 at isa? port @{$EnabledPort}[$EnabledCounter] net irq @{$EnabledIRQ}[$EnabledCounter] vector elintr\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "fe0") {

	     print FILE1 "device fe0 at isa? port @{$EnabledPort}[$EnabledCounter] net irq @{$EnabledIRQ}[$EnabledCounter] vector feintr\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "fea0") {

	     print FILE1 "device fea0\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "fpa0") {

	     print FILE1 "device fpa0\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "le0") {

	     print FILE1 "device le0 at isa? port @{$EnabledPort}[$EnabledCounter] net irq @{$EnabledIRQ}[$EnabledCounter] iomem 0xd00000 vector le_intr\n";
	   }

	   if (@{$EnabledCards}[$EnabledCounter] eq "sr0") {

	     print FILE1 "device sr0 at isa? port @{$EnabledPort}[$EnabledCounter] net irq @{$EnabledIRQ}[$EnabledCounter] iomem 0xd0000 vector srintr\n";
	   }
         }
      } else {

	  print FILE1 "$Line\n";
        }
    } else {

        print FILE1 "$Line\n";
      }
  }

  #
  # close both files
  #

  close (FILE);
  close (FILE1);

  #
  # Copy the changed one to the old one and remove the temp one
  #

  system ("cp $KernelPath/new$KernelName $KernelPath/$KernelName");

  system ("rm $KernelPath/new$KernelName");
}

}

sub change_audio_card_kernel {

my($KernelName, $Size, $TotalSize, $SoundDevice, $AllAudio, $AudioCards, $Port, $IRQ, $DMA) = @_;

#
# Open the $KernelName file to look if there are
# sound device and sound cards installed.
# If yes, skip that lines.
#

open (FILE, "$KernelPath/$KernelName");
open (FILE1, "> $KernelPath/new$KernelName");

while ($Line = <FILE>) {

  $PrintLine = 1;

  if (($Line =~ /snd0/) && ($Line !~ /^\#/)) {

    $Line = <FILE>;
  } else {

      print FILE1 "$Line";

      $PrintLine = 0;
    }

  for ($ArrayCounter = 0; $ArrayCounter < $TotalSize; $ArrayCounter++) {

     if ($Line =~ /device @{$AllAudio}[$ArrayCounter]/) {

       $Line = <FILE>;
       $ArrayCounter = 0;
     }
  }

  if ($PrintLine == 1) {

    print FILE1 "$Line";
  }
}

#
# close the files
#

close (FILE);
close (FILE1);

#
# Add the selected cards (if any) to the tempory file
# new$KernelName.
#

if ($Size > 0) {

  open (FILE1, ">> $KernelPath/new$KernelName");

  if (length($SoundDevice) > 0) {

    print FILE1 "\ncontroller\tsnd0\n";
  } else {

      print FILE1 "\n";
    }

  for ($ArrayCounter = 0; $ArrayCounter < $Size; $ArrayCounter++) {

     if (@{AudioCards}[$ArrayCounter] eq "sb0") {
       print FILE1 "device sb0\tat isa? port @{$Port}[$ArrayCounter] ";
       print FILE1 "irq @{$IRQ}[$ArrayCounter] conflicts ";
       print FILE1 "drq @{$DMA}[$ArrayCounter] vector sbintr\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "sbxvi0") {
	 print FILE1 "device sbxvi0\tat isa? drq @{$DMA}[$ArrayCounter]\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "sbmidi0") {
	 print FILE1 "device sbmidi0\tat isa? port @{$Port}[$ArrayCounter]\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "pas0") {
	 print FILE1 "device pas0\tat isa? port @{$Port}[$ArrayCounter] ";
         print FILE1 "irq @{$IRQ}[$ArrayCounter] ";
         print FILE1 "drq @{$DMA}[$ArrayCounter] vector pasintr\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "gus0") {
	 print FILE1 "device gus0\tat isa? port @{$Port}[$ArrayCounter] ";
         print FILE1 "irq @{$IRQ}[$ArrayCounter] ";
         print FILE1 "drq @{$DMA}[$ArrayCounter] vector gusintr\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "gusxvi0") {
	 print FILE1 "device gusxvi0\tat isa? port @{$Port}[$ArrayCounter] ";
         print FILE1 "irq @{$IRQ}[$ArrayCounter] ";
         print FILE1 "drq @{$DMA}[$ArrayCounter] flags 0x3 vector gusintr\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "mss0") {
	 print FILE1 "device mss0\tat isa? port @{$Port}[$ArrayCounter] ";
         print FILE1 "irq @{$IRQ}[$ArrayCounter] ";
         print FILE1 "drq @{$DMA}[$ArrayCounter] vector adintr\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "opl0") {
	 print FILE1 "device opl0\tat isa? port @{$Port}[$ArrayCounter] ";
         print FILE1 "conflicts\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "mpu0") {
	 print FILE1 "device mpu0\tat isa? port @{$Port}[$ArrayCounter] ";
         print FILE1 "irq @{$IRQ}[$ArrayCounter] drq @{$DMA}[$ArrayCounter]\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "uart0") {
	 print FILE1 "device uart0\tat isa? port @{$Port}[$ArrayCounter] ";
         print FILE1 "irq @{$IRQ}[$ArrayCounter] vector \"m6850intr\"\n";
     } elsif (@{$AudioCards}[$ArrayCounter] eq "pca0") {
	 print FILE1 "device pca0\tat isa? port IO_TIMER1 tty\n";
       }
   }

   close (FILE1);
}

#
# Copy the changed one to the old one and remove the temp one
#

system ("cp $KernelPath/new$KernelName $KernelPath/$KernelName");

system ("rm $KernelPath/new$KernelName");

}

sub change_filesystems_kernel {

my($KernelName, $Size, $TotalSize, $AllFileSys, $FileSystems) = @_;

$LineCounter = 0;
$FirstFileSystemPosition = 0;

#
# Open the both files
#

open (FILE, "$KernelPath/$KernelName");
open (FILE1, "> $KernelPath/new$KernelName");

#
# Skip all filesystems support in the kernel $KernelName
#

while ($Line = <FILE>) {

  $LineCounter++;

  if ($Line =~ /options/) {

    for ($ArrayCounter = 0; $ArrayCounter < $TotalSize; $ArrayCounter++) {

       if ($Line =~ /@{$AllFileSys}[$ArrayCounter]/) {

	 $Line = <FILE>;
	 $ArrayCounter = 0;

	 if ($FirstFileSystemPosition == 0) {

	   $FirstFileSystemPosition = $LineCounter;
         }

       }
    }

    print FILE1 "$Line";

  } else {

      print FILE1 "$Line";
    }
}

#
# close both files
#

close (FILE);
close (FILE1);

#
# Copy the changed one to the old one and remove the temp one
#

system ("cp $KernelPath/new$KernelName $KernelPath/$KernelName");

system ("rm $KernelPath/new$KernelName");

#
# If user has enabled filesystems, write them in the kernel
# $KernelName op the same position as the previous filesystem
# 

$LineCounter = 0;

if ($Size > 0) {

  #
  # open the files
  #

  open (FILE, "$KernelPath/$KernelName");
  open (FILE1, "> $KernelPath/new$KernelName");

  #
  # Jump to the the known position in the file
  #

  while ($Line = <FILE>) {

    $LineCounter++;

    #print "$FirstFileSystemPosition\n";

    if ($LineCounter == $FirstFileSystemPosition) {

      for ($ArrayCounter = 0; $ArrayCounter < $Size; $ArrayCounter++) {

         if (@{$FileSystems}[$ArrayCounter] eq "FFS") {

           print FILE1 "options\t\tFFS\t\t\t\#Fast filesystem\n";
         } elsif (@{$FileSystems}[$ArrayCounter] eq "NFS") {

	     print FILE1 "options\t\tNFS\t\t\t\#Network File System\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "NQNFS") {

	     print FILE1 "options\t\tNQNFS\t\t\t\#Enable NQNFS lease ";
	     print FILE1 "checking\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "CD9660") {

	     print FILE1 "options\t\t\"CD9660\"\t\t\#ISO 9660 filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "FDESC") {

	     print FILE1 "options\t\tFDESC\t\t\t\#File descriptor ";
	     print FILE! "filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "KERNFS") {

	     print FILE1 "options\t\tKERNFS\t\t\t\#Kernel filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "LFS") {

	     print FILE1 "options\t\tLFS\t\t\t\#Log filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "MFS") {

	     print FILE1 "options\t\tMFS\t\t\t\#Memory File System\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "MSDOSFS") {

	     print FILE1 "options\t\tMSDOSFS\t\t\t\#MS DOS File System\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "NULLFS") {

	     print FILE1 "options\t\tNULLFS\t\t\t\#NULL filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "PORTAL") {

	     print FILE1 "options\t\tPORTAL\t\t\t\#Portal filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "PROCFS") {

	     print FILE1 "options\t\tPROCFS\t\t\t\#Process filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "UMAPFS") {

	     print FILE1 "options\t\tUMAPFS\t\t\t\#UID map filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "UNION") {

	     print FILE1 "options\t\tUNION\t\t\t\#Union filesystem\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "NFS_NOSERVER") {

	     print FILE1 "options\t\tNFS_NOSERVER\t\t\#Disable the NFS-";
	     print FILE1 "server code\n";
	 } elsif (@{$FileSystems}[$ArrayCounter] eq "DEVFS") {

	     print FILE1 "options\t\tDEVFS\t\t\t\#Devices filesystem\n";
           }
      }

      print FILE1 "$Line";

    } else {

	print FILE1 "$Line";
      }
  }

  #
  # close the file
  #

  close (FILE);
  close (FILE1);

  #
  # Copy the changed one to the old one and remove the temp one
  #

  system ("cp $KernelPath/new$KernelName $KernelPath/$KernelName");

  system ("rm $KernelPath/new$KernelName");
}

}

# changekernel.pm

