package check;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(check_passwd check_name check_virtual_terminals check_virtual_console check_fddi_sppp);

#
# check password or check name or check virtual terminals
# or check virtual console file or check fddi and/or sppp
# support for FreeBSB
# (C) 1997 by Wouter de Boer
#
# Usage:
#    check_passwd arg1, arg2, arg3
#
#	arg1: FirstPassword
#       arg2: SecondPassword
#	arg3: User Name
#
#    check_name arg1, arg2
#	
#	arg1: Orginal Users
#	arg2: Invited Users
#
#    check_virtual_terminals arg1, arg2, arg3
#	
#	arg1: Size of Orginal Virtual Terminals
#	arg2: Orginal Virtual Terminals
#	arg3: New Virtual Terminals
#
#    check_virtual_console arg1, arg2
#
#	arg1: Orginal Virtual Console
#	arg2: New Virtual Console
#
#    check_fddi_sppp arg1, arg2, arg3
#
#       arg1: FDDI 
#       arg2: SPPP
#       arg3: Kernel Name
#

sub check_passwd {

use verify;

my($FirstPasswd) = $_[0];
my($SecondPasswd) = $_[1];
my($OrginalUser) = $_[2];

if ($FirstPasswd eq $SecondPasswd) {
  
  $Passwd = $FirstPasswd;

  #
  # Check if password is new or still the same
  #

  if ((length($Passwd) > 0) && (length($OrginalUser) > 0)) {
    
    $ValidPass = verify_passwd $OrginalUser, $Passwd;

    if ($ValidPass == 0) {

      $Passwd = "";
    }
  }

} else {

    $Passwd = "";
}

return $Passwd;

}

sub check_name {

my($OrginalName, @InvitedName) = @_;

#
# Store all users into array
#

@OrgName = split(/,/, $OrginalName);

$Size = scalar @InvitedName;
$Size1 = scalar @OrgName;
$Same = 0;

if ($Size == $Size1) {

  for ($FirstCounter = 0; $FirstCounter < $Size; $FirstCounter++) {

      for ($SecondCounter = 0; $SecondCounter < $Size1; $SecondCounter++) {

         if (@InvitedName[$FirstCounter] eq @OrgName[$SecondCounter]) {

	   $Same++;
	 }
      }
  }
  if ($Same == $Size1) {

    $AllSame = 1;
  } else {

      $Allsame = 0;
    }
} else {

    $AllSame = 0;
  }

return $AllSame;

}

sub check_virtual_terminals {

my($OrgSize, $OrgVirTermArray, @VirTermArray) = @_;

my($Counter) = 0;

while ($Counter < $OrgSize) {

   if (((@{$OrgVirTermArray}[$Counter]) =~ /$VirTermArray[0]/) && (length($VirTermArray[0]) > 0)) {

       shift @VirTermArray;
       splice (@{$OrgVirTermArray}, $Counter, 1);
       $OrgSize = scalar @{$OrgVirTermArray};
       $Counter = 0;
   } else {
 
       $Counter++;
     }
}

#
# Obtain the size of both virtual arrays
#

my($SizeVirtual) = scalar @VirTermArray;
$OrgSize = scalar (@{$OrgVirTermArray});

return $SizeVirtual, $OrgSize, $OrgVirTermArray, \@VirTermArray;

}

sub check_virtual_console {

my($OrgVirCons, @VirCons) = @_;

my($Size) = scalar @{$OrgVirCons};

my($ArrayCounter, $Counter) = 0;

my($ConsoleUnchanged) = 1;

while ($ArrayCounter < $Size) {

  $TempCounter = $Counter;  
  if ($Counter == 10) {

    $Counter = a;
  } elsif ($Counter == 11) {

      $Counter = b;
    }
  $Number = @{$OrgVirCons}[$ArrayCounter];
  $Number =~ s/ttyv//;

  if ($Number eq "a") {

    $Number = 10;
  } elsif ($Number eq "b") {

      $Number = 11;
    }

  if (@{$OrgVirCons}[$ArrayCounter] =~ /ttyv$Counter/) {

    if (@{$OrgVirCons}[$ArrayCounter] !~ /$VirCons[$Number]/) {

      if ((@{$OrgVirCons}[$ArrayCounter] =~ /xterm/) && ($VirCons[$Number] !~ /xterm/)) {

        if ($ConsoleUnchanged != 0) {

	     $ConsoleUnchanged = 1;
	}
      } else {

          $ConsoleUnchanged = 0;
        }
    }
    $ArrayCounter++;
    $Counter = $TempCounter;
    $Counter++;
  } else {

      $Counter = $TempCounter;
      $Counter++;
    }
}

return $ConsoleUnchanged;

}

sub check_fddi_sppp {

#
# Obtain variable names
#

my($FDDI, $SPPP, $KernelName) = @_;

#
# Variable used in this function
#

$KernelPath = "/usr/src/sys/i386/conf";

#
# Open the file and search for fddi and/or fppp
#

open (FILE, "$KernelPath/$KernelName");

while ($Line = <FILE>) {

  if (($Line =~ /pseudo-device/) && ($Line =~ /fddi/) && ($Line !~ /^\#/)) {

    $FDDI = "No";
  }

  if (($Line =~ /pseudo-device/) && ($Line =~ /sppp/) && ($Line !~ /^\#/)) {

    $SPPP = "No";
  }
}

#
# Close the file
#

close (FILE);

#
# Return with both values
#

return ($FDDI, $SPPP);

}

# check.pm



