package returnfilesystems;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(return_filesystems);

#
# Return all filesystems which are supported by a 
# selected kernel
# (C) 1997 by Wouter de Boer
#
# Usage:
#    return_filesystems arg1
#
#	arg1: Kernel Name
#

use removeleadingspace;

sub return_filesystems {

my($KernelName) = @_;

#
# Variables used in this program
#

$KernelPath = "/usr/src/sys/i386/conf";
my(@AllFileSystemsArray) = ('FFS', 'NFS', 'NQNFS', 'NFS_NOSERVER', 'CD9660', 'FDESC', 'KERNFS', 'LFS', 'MFS', 'MSDOSFS', 'NULLFS', 'PORTAL', 'PROCFS', 'UMAPFS', 'UNION', 'DEVFS');
$Size = scalar @AllFileSystemsArray;
$SupportedCounter = $Disabled = 0;
$KernFS = 0;
$NullFS = 0;

#
# Open the kernel file for reading
#

open (FILE, "$KernelPath/$KernelName");

#
# Selerct all filesystems in an array
#

while ($Line = <FILE>) {

  $Disabled = 0;

  $Line = remove_leading_space $Line; 

  if (($Line =~ /^options/) || (($Line =~ /^\#/) && ($Line =~ /options/))) {

    for ($ArrayCounter = 0; $ArrayCounter < $Size; $ArrayCounter++) {

       if ($Line =~ /$AllFileSystemsArray[$ArrayCounter]/) {

	 chop($Line);

	 if ($Line =~ /^\#/) {

	   $Disabled = 1;
         }

	 #
	 # Because the fact, the search pattern fails for KERNFS and
	 # NULLFS, both filesystems can't be manupilled automattically
	 #

	 if ($Line =~ /KERNFS/) {

	   $Line =~ s/options//;
	   $Line =~ s/KERNFS//;
	   $Line =~ s/\#//g;
	   $Line =~ s/\"//g;
	   $KernFS = 1;
         } elsif ($Line =~ /NULLFS/) {

	     $Line =~ s/options//;
	     $Line =~ s/NULLFS//;
	     $Line =~ s/\#//g;
	     $Line =~ s/\"//g;
	     $NullFS = 1;
	   } else {

	       $Line =~ s/options//;
	       $Line =~ s/$AllFileSystemsArray[$ArrayCounter]//;
	       $Line =~ s/\#//g;
	       $Line =~ s/\"//g;
	     }

	 $Line = remove_leading_space $Line;

	 if ($Disabled == 1) {

	   $Hulp = "\#";
	   $Hulp .= "$Line";
	   $Line = $Hulp;
         }

	 if ($KernFS == 1) {

	   $SupportedFileSystemsArray[$SupportedCounter] = "KERNFS";
	   $KernFS = 0;

	   for ($Counter = 0; $Counter < $Size; $Counter++) {

	      if ($AllFileSystemsArray[$Counter] eq "KERNFS") {

		splice (@AllFileSystemsArray, $Counter, 1);
		$Size = scalar @AllFileSystemsArray;
	      }
	   }
         } elsif ($NullFS == 1) {

	     $SupportedFileSystemsArray[$SupportedCounter] = "NULLFS";
	     $NullFS = 0;

	     for ($Counter = 0; $Counter < $Size; $Counter++) {

	       if ($AllFileSystemsArray[$Counter] eq "NULLFS") {

		 splice (@AllFileSystemsArray, $Counter, 1);
		 $Size = scalar @AllFileSystemsArray;
	       }
	     }
	   } else {

	       $SupportedFileSystemsArray[$SupportedCounter] = $AllFileSystemsArray[$ArrayCounter];

	       splice (@AllFileSystemsArray, $ArrayCounter, 1);
	       $Size = scalar @AllFileSystemsArray;
	     }

	 $DescriptionsArray[$SupportedCounter++] = $Line;
       }
    }
  }
}

#
# Close the file
#

close (FILE);

#
# Return with supported filesystems and the size
#

return ($Size, $SupportedCounter, \@SupportedFileSystemsArray, \@AllFileSystemsArray, \@DescriptionsArray);

}

# returnfilesystems.pm
