package returnsysconfigsettings;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(return_sysconfig_settings return_ipaddress_netmask return_short_networkcard_name return_netmask);

use removeleadingspace;

#
# Location of sysconfig file
#

$SysConfigPath = "/etc/sysconfig";

sub return_sysconfig_settings {

#
# Return the needed settings of the $SysConfigPath
# (C) 1997 by Wouter de Boer
#
# Usage:
#    return_sysconfig_settings arg1 .... argN
#
#	arg1 .... argN sysconfig variable names
#

my (%AssocArray) = @_;

#
# open the $SysConfigPath for reading
#

open (FILE, $SysConfigPath);

while ($Line = <FILE>) {

  chop($Line);

  $Line = remove_leading_space $Line;

  if ($Line !~ /^#/) {

    @SubString = split(/=/, $Line);

    if (exists($AssocArray{$SubString[0]})) {

      $SubString[1] =~ s/"//g;
      $AssocArray{$SubString[0]} = $SubString[1];
    }
  }
}

#
# close the file
#

close (FILE);

#
# return with all settings
#

return (%AssocArray);

}

sub return_ipaddress_netmask {

#
# Return the IP Address and the netmask number of a certain network card.
# (C) 1997 by Wouter de Boer
#
# Usage:
#    return_ipaddress_netmask
#

my($Card) = $_[0];

#
# open the $SysConfigPath for reading
#

open (FILE, $SysConfigPath);

while ($Line = <FILE>) {

  chop($Line);

  if ($Line =~ /^ifconfig_$Card/) {

    $Line =~ s/\"//g;
    @TempArray = split(/=/, $Line);
    @TempArray1 = split(/\s+/, $TempArray[1]);
    if ($TempArray1[0] =~ /inet/) {

      $IPAddress = $TempArray1[1];
      $Netmask = $TempArray1[3];
    } else {

	$IPAddress = $TempArray1[3];
        $Netmask = $TempArray1[1];
      }
  }
}

close (FILE);

#
# if $Netmask is hex, convert to decimal value
# 

$Netmask = &convert_hex_to_decimal($Netmask);

#
# return with IP Address and netmask
#

return ($IPAddress, $Netmask);

}

sub return_short_networkcard_name {

#
# return all short networkcard names in FreeBSD
# (c) 1997 by Wouter de Boer 

#
# All network names in array
# 

my(@NetworkArray) = ('ar0', 'cx0', 'de0', 'de1', 'ed0', 'eg0', 'el0', 'ep0', 'fe0', 'fea0', 'fpa0', 'fpx0', 'ie0', 'ix0', 'le0', 'lnc0', 'sr0', 'vx0', 'ze0', 'zp0');

#
# Obtain the size of the array
#

my($Size) = scalar @NetworkArray;

#
# open the $SysConfigPath for reading
#

open (FILE, $SysConfigPath);

while ($Line = <FILE>) {

  chop ($Line);

  $Line = remove_leading_space $Line;

  if ($Line =~ /^network_interfaces=/) {

    my($InterFace) = $Line;

    for ($ArrayCounter = $Counter = 0; $Counter < $Size; $Counter++) {

       if ($InterFace =~ /$NetworkArray[$Counter]/) {

         $Card[$ArrayCounter++] = $NetworkArray[$Counter]; 
       }
    }
  }
}

#
# Close the file
#

close (FILE);

return @Card;

}

sub return_netmask {

#
# Return the the first netmask number if available. This netmask number will be
# used by new networkcards.
# (C) 1997 by Wouter de Boer
#
# Usage:
#    return_netmask
# 

#
# open the $SysConfigPath for reading
#

open (FILE, $SysConfigPath);

while ($Line = <FILE>) {

  chop($Line);

  if (($Line =~ /^ifconfig_/) && ($Line =~ /netmask/)) {

    $Line =~ s/\"//g;
    @TempArray = split(/=/, $Line);
    @TempArray1 = split(/\s+/, $TempArray[1]);
    if ($TempArray1[0] =~ /inet/) {

      $Netmask = $TempArray1[3];
    } else {

        $Netmask = $TempArray1[1];
      }
  }
}

close (FILE);

#
# if $Netmask is hex, convert to decimal value
#

$Netmask = &convert_hex_to_decimal($Netmask);

#
# return with IP Address and netmask
#

return ($Netmask);

}

sub convert_hex_to_decimal {

my ($Netmask) = $_[0];

#
# if hex values are used, convert these hex values to decimal values
#

if ($Netmask eq "0xff000000") {

  $Netmask = "255.0.0.0";
} elsif ($Netmask eq "0xffff0000") {

    $Netmask = "255.255.0.0";
  } elsif ($Netmask eq "0xffffff00") {

      $Netmask = "255.255.255.0";
    }

return $Netmask;

}

# returnsysconfigsettings.pm





